@@ -1,5 +1,4 @@
 from collections.abc import Iterable
-from collections import OrderedDict
 from bs4 import BeautifulSoup
 from PIL import Image
 from ruamel import yaml
@@ -778,6 +777,63 @@ def read_table(file_name, delimiter=',', ignore_lines=True):
         DEBUG_LEVEL = temp_debug
     return output
 
+def scan_dir(folder=None, regex=None, recursive=True):
+    """Scans the directory :code:`folder` recursively and returns all files and sub-folders as two lists.
+    
+    :param str folder: A directory to scan. Defaults to the current working directory.
+    :param str|re regex: If given, this function will only return files whose full paths match this regex. This has no effect on sub-folders.
+    :param bool recursive: If False, this function will not scan sub-folders recursively. You might be better off using :code:`os.walk()` if you set this to False.
+    
+    :returns: files and subfolders as two lists in that order. :code:`return files, subfolders`
+    
+    :raises InvalidCharacterError: If :code:`folder` contains invalid characters.
+    :raises FileExistsError: If :code:`folder` is not a folder.
+    :raises FileNotFoundError: If :code:`folder` does not exist.
+    
+    :note: Any slashes (:code:`/`) in file paths will be replaced with backslashes (:code:`\\`) in the output, and this is what regexes match with.
+    
+    :since 1.4:
+    """
+    if not folder:
+        folder = os.getcwd()
+    folder = folder.replace('/', '\\')
+    
+    if recursive:
+        debug('Scanning folder "{}" recursively', 2, folder, pre='[FUNC] ')
+    else:
+        debug('Scanning folder "{}"', 2, folder, pre='[FUNC] ')
+    
+    def scan(folder, r):
+        files, folders = [], []
+        for f in os.scandir(folder):
+            if f.is_dir():
+                folders.append(f.path)
+            elif r == None or r.fullmatch(f.path):
+                files.append(f.path)
+        return files, folders
+    
+    if isinstance(regex, str):
+        rext = re.compile(regex)
+    else:
+        rext = regex
+    
+    if INVALID_CHARS_NO_SLASH.search(folder) is not None:
+        raise InvalidCharacterError('Folder "' + folder + '" contains invalid characters.')
+    if os.path.exists(folder):
+        if os.path.isdir(folder):
+            files, folders = scan(folder, rext)
+            if recursive:
+                for f in folders:
+                    nf, no = scan(f, rext)
+                    files.extend(nf)
+                    folders.extend(no)
+            return files, folders
+        else:
+            raise FileExistsError('"' + folder + '" is not a folder.')
+    else:
+        raise FileNotFoundError('Could not find folder "' + folder + '"')
+    
+
 def make_dir(folder, change=False, overwrite=False):
     """Makes a folder at :code:`folder`, along will all parent folders.
     
@@ -974,6 +1030,63 @@ def write_table(file_name, table, delimiter=',', force=False):
     finally:
         TABULATION -= 1
 
+def replace(files, old, new, regex=None, ignore_errs=True):
+    """Replaces all instances of :code:`old` with :code:`new` in :code:`files`.
+    
+    :param [str,....]|str files: A string or list of strings pointing to files to have their contents changed.
+    :param str|re old: A regex or string to replace. If this is a string, it will not be treated as a regex. Use :code:`re.compile()` to pass in a regex object.
+    :param str new: A string of text to replace with.
+    :param str|re regex: If given, this function will only replace in files whose full paths match this regex. It must be a real regex (i.e., you can't use "*" in place of ".*")!
+    :param bool ignore_errs: If False, this function will throw an error at the end if :code:`files` had invalid files.
+    
+    :raises: A bulk ValueError if :code:`files` has any invalid files and :code:`ignore_errs` is False. Errors will be thrown at the end of the function after everything that could be replaced was replaced.
+    
+    :since 1.4:
+    """
+    if isinstance(files, str):
+        file_names = [files]
+    else:
+        file_names = files
+    
+    debug('Replacing contents in {} files', 2, len(file_names), pre='[FUNC] ')
+    
+    errs = []
+    global DEBUG_LEVEL, TABULATION
+    temp_debug, DEBUG_LEVEL = DEBUG_LEVEL, 0
+    try:
+        if isinstance(regex, str):
+            rext = re.compile(regex)
+        else:
+            rext = regex
+        
+        for file_name in file_names:
+            if rext != None and not rext.fullmatch(file_name):
+                continue
+            try:
+                with codecs.open(file_name, 'r', 'utf-8') as file:
+                    ct = file.read()
+                if isinstance(old, str):
+                    ct = ct.replace(old, new)
+                else:
+                    ct = old.sub(new, ct)
+                with codecs.open(file_name, 'w', 'utf-8') as file:
+                    file.write(ct)
+            except Exception as e:
+                errs.append(e)
+    finally:
+        DEBUG_LEVEL = temp_debug
+    
+    if errs:
+        msg = '{} files had problems'.format(len(errs))
+        if ignore_errs:
+            TABULATION += 1
+            try:
+                debug(msg, 2, pre='[ERR] ')
+            finally:
+                TABULATION -= 1
+        else:
+            raise ValueError(msg)
+
 def hash256(obj):
     """Serializes :code:`obj` and then returns a sha256 HASH object of it.
     
@@ -1092,9 +1205,11 @@ def curate(items, curation_class, use_title=False, save=False, ignore_errs=False
                 errs.append((item, e, data))
             except KeyboardInterrupt as e:
                 debug('Interrupt received, terminating curation process (but not clearing save)', 1)
-                isave = False
-                errs.append((item, e, data))
-                break
+                if ignore_errs:
+                    errs.append((item, e, data))
+                    return errs
+                else:
+                    return None
             except Exception as e:
                 if not ignore_errs:
                     raise e
@@ -1198,9 +1313,11 @@ def curate_regex(items, links, use_title=False, save=False, ignore_errs=False, o
                         errs.append((item, e, data))
                     except KeyboardInterrupt as e:
                         debug('Interrupt received, terminating curation process (but not clearing save)', 1)
-                        isave = False
-                        errs.append((item, e, data))
-                        break
+                        if ignore_errs:
+                            errs.append((item, e, data))
+                            return errs
+                        else:
+                            return None
                     except Exception as e:
                         if not ignore_errs:
                             raise e
@@ -1237,7 +1354,10 @@ def load(curation):
     
     data = yaml.round_trip_load(read(os.path.join(curation, 'meta.yaml')))
     c = Curation()
-    c.meta = data
+    c.meta = dict(data)
+    c.meta['Additional Applications'] = dict(c.meta['Additional Applications'])
+    for app in c.meta['Additional Applications']:
+        c.meta['Additional Applications'][app] = dict(c.meta['Additional Applications'][app])
     
     folder = curation.replace('\\', '/')
     if '/' in folder:
@@ -1338,30 +1458,30 @@ class Curation:
         :raises TypeError: If :code:`curation` is not an instance of :class:`Curation`.
         """
         if not curation:
-            self.meta = OrderedDict([
-                ('Title', None),
-                ('Alternate Titles', None),
-                ('Library', 'arcade'),
-                ('Series', None),
-                ('Developer', None),
-                ('Publisher', None),
-                ('Play Mode', 'Single Player'),
-                ('Release Date', None),
-                ('Version', None),
-                ('Languages', 'en'),
-                ('Extreme', 'No'),
-                ('Tags', None),
-                ('Source', None),
-                ('Platform', 'Flash'),
-                ('Status', 'Playable'),
-                ('Application Path', FLASH),
-                ('Launch Command', None),
-                ('Game Notes', None),
-                ('Original Description', None),
-                ('Curation Notes', None),
-                ('Additional Applications', OrderedDict())
-            ])
-            """An ordered dictionary containing all metadata for the curation. While you can modify it directly, it is recommended that you use :func:`Curation.set_meta()` and :func:`Curation.get_meta()` instead."""
+            self.meta = {
+                'Title': None,
+                'Alternate Titles': None,
+                'Library': 'arcade',
+                'Series': None,
+                'Developer': None,
+                'Publisher': None,
+                'Play Mode': 'Single Player',
+                'Release Date': None,
+                'Version': None,
+                'Languages': 'en',
+                'Extreme': 'No',
+                'Tags': None,
+                'Source': None,
+                'Platform': 'Flash',
+                'Status': 'Playable',
+                'Application Path': FLASH,
+                'Launch Command': None,
+                'Game Notes': None,
+                'Original Description': None,
+                'Curation Notes': None,
+                'Additional Applications': {}
+            }
+            """A dictionary containing all metadata for the curation. While you can modify it directly, it is recommended that you use :func:`Curation.set_meta()` and :func:`Curation.get_meta()` instead."""
             self.args = {}
             """A dictionary containing all arguments passed in through :func:`Curation.set_meta()` that do not map to any metadata. You can use this to pass in extra information that you want to use in :func:`Curation.parse()` or other methods for custom classes."""
             
@@ -1446,10 +1566,10 @@ class Curation:
         debug('Adding app "{}" to curation {}', 2, heading, str(self), pre='[FUNC] ')
         if heading.lower() in Curation.RESERVED_APPS:
             raise ValueError('You cannot create an additional app with the name"' + heading + '"')
-        self.meta['Additional Applications'][heading] = OrderedDict([
-            ('Application Path', path),
-            ('Launch Command', launch)
-        ])
+        self.meta['Additional Applications'][heading] = {
+            'Application Path': path,
+            'Launch Command': launch
+        }
     
     def add_ext(self, folder):
         """Add extras from folder.
